#define SIZE 256
#define MAXCOUNT 150

union color
{
  struct
  {
    unsigned char r;
    unsigned char g;
    unsigned char b;
    unsigned char a;
  };
  unsigned int c;
} color[SIZE][SIZE];

typedef struct vec2
{
  float x;
  float y;
} vec2;

static inline union color colorize(unsigned int a)
{
    int i = ((float)a / (float)MAXCOUNT) * 6.0;
    float f = (((float)a / (float)MAXCOUNT) * 6.0) - i;
    float q = 1.0 - f;
    float t = 1.0 - ((1.0 - f) * 1.0);

    switch(i % 6)
    {
        case 0: return (union color){ .r = 0xFF    , .g = t * 0xFF, .b = 0x00    , .a = 0xFF };
        case 1: return (union color){ .r = q * 0xFF, .g = 0xFF    , .b = 0x00    , .a = 0xFF };
        case 2: return (union color){ .r = 0x00    , .g = 0xFF    , .b = t * 0xFF, .a = 0xFF };
        case 3: return (union color){ .r = 0x00    , .g = q * 0xFF, .b = 0xFF    , .a = 0xFF };
        case 4: return (union color){ .r = t * 0xFF, .g = 0x00    , .b = 0xFF    , .a = 0xFF };
        case 5: return (union color){ .r = 0xFF    , .g = 0x00    , .b = q * 0xFF, .a = 0xFF };
        default: return (union color){ .r = 0, .g = 0, .b = 0, .a = 0xFF };
    };
}

int game(float time);
int game(float time)
{
  for (int y = 0;y < SIZE;y++)
  {
    for (int x = 0;x < SIZE;x++)
    {
      vec2 c = { time / 100.0, time / 230.0 };
      vec2 z = (vec2){ .x = ((float)x / SIZE - 0.5) * 2.5, .y = ((float)y / SIZE - 0.5) * 2.5 };
      unsigned int count = 0;
      while (z.x * z.x + z.y * z.y < 2*2 && count < MAXCOUNT)
      {
        float t = z.x * z.x - z.y * z.y + c.x;
        z.y = 2 * z.x * z.y + c.y;
        z.x = t;
        count++;
      }
      color[y][x] = colorize(count+(MAXCOUNT - 42));
    }
  }
  return 0;
}
